

-- License GPLv2: http://www.gnu.org/licenses/gpl-2.0.html
--
-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License version 2 as
-- published by the Free Software Foundation.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.


-- #######################################################
-- ## 							                       	##
-- ## 	performance measurement 	                  	##
-- ## 	Script by 	strgaltdel 	                       	##
-- ##   Rev  1.0 	Jan. 2017                          	##
-- ## 							                       	##
-- ## 	hardware	Taranis (9d/9d+/9e)                 ##
-- ##   openTx		2.2 RC8				               	##
-- ## 	unow											##
-- ## 							                       	##
-- ## feel free to customize by yourself			   	##
-- #######################################################   


-- ##########################################################      customizing start

local sw_record = "sh"		-- start record switch
local Tmin = 10				-- recording duration (in seconds) for one measurement (minimum)
local Tmax = 25				-- recording duration (in seconds) for one measurement (maximum)
local setTime = "s2"		-- pod to select recording duration between min & max
local max_cyc = 4			-- number of cycles
local Vdelta = 3			-- max deviation (in percent) from avg speed until measurement will be stopped

-- Telemetry variable: Vario
local l_Vclimbrate = 	"VSpd"

-- Telemetry variable: Altitude
local l_Alt =	"Alt"

-- Telemetry variable: Airspeed
local l_Airspeed =	"ASpd"

-- ##########################################################      customizing end


-- actual record situation
local record_actual = 0

-- get actual model
local plane = model.getInfo()


-- variables which define resolution & precision
local metric = 1.852			-- needed for some km/ miles conversions
local Asp_factor = 10			-- airspeed measured with one decimal , so multiplicate value by 10
--local V_factor = 100			-- Vrate measured with two decimals , so multiplicate value by 100


local rt_meas = {}				-- measurement array
local l_time					-- actual time

local Asp_min = 0				-- airspeed min/max
local Asp_max = 0		
local i,n

-- init some more values
local avg_speed = 1				-- sum of average 		
local num_samples = 1			-- number of samples
-- start with cycle #1
local cycle = 1					-- actual meas. cycle
local pt_status = 0				-- tmp
local delta_t					-- measurement in seconds





--------------
-- Rounding --
--------------
local function rnd(v,d)
	if d then
	 return math.floor((v*10^d)+0.5)/(10^d)
	else
	 return math.floor(v+0.5)
	end
end



--------------------
-- display values --
--------------------

local function arrayPrint()
	local x_coord = 21		-- first array col
	local y_coord = 17		-- first array line
	local y_high = 8
	local x1_delta = 23
	local x2_delta = x1_delta + 22
	local x3_delta = x2_delta + 22
	local l_style = SMLSIZE
	local l_style_header = MIDSIZE+INVERS 
	local l_style_value = DBLSIZE + INVERS
	local l_date = getDateTime()
	local delta_alt
	local tmp = getValue("Alt")
	
	
	

		lcd.clear()
		-- display header data
		lcd.drawText(1, 0, plane.name , SMLSIZE)
		lcd.drawText(58, 0, "  Performance  ",l_style_header)

		-- display array header	
		lcd.drawText(x_coord+2, y_coord, 'Aspd' ,RIGHT +l_style + INVERS)
		lcd.drawText(x_coord+x1_delta+1, y_coord, 'sink' ,RIGHT +l_style + INVERS)
		lcd.drawText(x_coord+x2_delta, y_coord, ' l/d' ,RIGHT +l_style + INVERS)
		lcd.drawText(x_coord+x3_delta-2, y_coord, 'dV' ,RIGHT +l_style + INVERS)
		lcd.drawText(x_coord+x3_delta +18,y_coord, "dA",RIGHT +l_style + INVERS)
		lcd.drawText(x_coord+x3_delta +31,y_coord, " t",RIGHT +l_style + INVERS)
		
		-- display array-block 
		for i=1,max_cyc do
			delta_alt = rt_meas[i].altStart - rt_meas[i].altEnd
			lcd.drawNumber(x_coord, y_coord + i * y_high, rt_meas[i].speed*Asp_factor,RIGHT + PREC1 + l_style)											-- avg speed
			lcd.drawNumber(x_coord + x1_delta, y_coord + i * y_high, delta_alt*100/rt_meas[i].dura , RIGHT + PREC2 + l_style)							-- avg Vsink

			
			if record_actual == 1 and i == cycle then		-- measurement time
				lcd.drawNumber(x_coord + x3_delta + 43, y_coord + i * y_high, delta_t -  ((l_time.min*60 + l_time.sec) - rt_meas[cycle].start) ,RIGHT + l_style + INVERS)  -- countdown
			end

			if delta_alt ~= 0 then
				lcd.drawNumber(x_coord + x2_delta, y_coord + i * y_high, rt_meas[i].ldratio*10 , RIGHT + PREC1 + l_style)								-- avg ld ratio
				lcd.drawNumber(x_coord + x3_delta, y_coord + i * y_high, (rt_meas[i].vmax - rt_meas[i].vmin)*Asp_factor , RIGHT +PREC1 + l_style)  		-- deltaV
				lcd.drawNumber(x_coord + x3_delta+18, y_coord + i * y_high, (rt_meas[i].altStart - rt_meas[i].altEnd) *10, RIGHT +PREC1+ l_style) 		-- deltaAlt
				lcd.drawNumber(x_coord + x3_delta+32, y_coord + i * y_high, rt_meas[i].dura , RIGHT + l_style) 											-- duration
				else
				lcd.drawText(x_coord + x2_delta, y_coord + i * y_high, "**", RIGHT + l_style)
			end
		end	
		lcd.drawNumber(200, 1 ,Tmin +  (0.5+(getValue(setTime)/2048))*(Tmax-Tmin),RIGHT  + SMLSIZE)				-- duration										-- info:time

end	

----------------------
--   disp average 	--
--     values   	--
----------------------

local function disp_average(x_coord,y_coord)

	local m_speed = 0
	local m_sink = 0
	local m_ld = 0
	
	lcd.drawRectangle(x_coord ,y_coord -2,70,48)
	for i = 1,cycle-1 do
		m_speed = m_speed + rt_meas[i].speed
		m_sink = m_sink + (rt_meas[i].altStart - rt_meas[i].altEnd)/rt_meas[i].dura
		m_ld = m_ld + rt_meas[i].ldratio
	end
	
	m_speed = m_speed/(cycle-1)
	m_sink = m_sink / (cycle-1)
	m_ld = m_ld / (cycle-1)
	
	lcd.drawText(170,y_coord +2,"Sinken",RIGHT + SMLSIZE )
	lcd.drawText(170,y_coord +17,"Gleiten",RIGHT + SMLSIZE )
	lcd.drawText(170,y_coord +34,"Speed",RIGHT + SMLSIZE )
	lcd.drawNumber(200, y_coord , m_sink*100, RIGHT + PREC2 + MIDSIZE + INVERS)
	lcd.drawNumber(200, y_coord +15, m_ld*10, RIGHT + PREC1 + MIDSIZE + INVERS)	
	lcd.drawNumber(200, y_coord +32, m_speed*10, RIGHT + PREC1 + MIDSIZE + INVERS)
end



----------------------
--   some tones 	--
----------------------

local function beep_start()
	playTone(1600,100,0)
end


local function beep_ongoing()
	playTone(2000,60,0)
end


local function beep_finished()
	playTone(2100,250,0)
end


local function beep_abort()
	playTone(170,300,0)
end


----------------------
--   init values 	--
--     at begin   	--
--	   of cycle		--
----------------------
local function init_values(speed)
	record_actual = 1																	-- proclaim record active
	rt_meas[cycle].altStart = getValue("Alt")											-- altitude at start
	delta_t = Tmin +  (0.5+(getValue(setTime)/2048))*(Tmax-Tmin)						-- calc duration
	rt_meas[cycle].dura = delta_t														-- set duration into array
	l_time = getDateTime()
	rt_meas[cycle].start = tmp_seconds													-- set timestamp at start
	Asp_max = speed																		-- init some values
	Asp_min = speed
	avg_speed = speed	
end


----------------------
--   reset values 	--
--   after cycle  	--
----------------------
local function reset_values()
	Asp_max = 0
	Asp_min = 0
	avg_speed = 0
	num_samples = 1	
	delta_alt = 0
	record_actual = 0
	rt_meas[cycle] =  {start =0, altStart=0, altEnd=0, speed=0, ldratio=0, vmin=0, vmax=0, dura=1}
end

----------------------
--    store data 	--
--  if successfully --
--	  finished		--
----------------------
local function meas_finished()
	rt_meas[cycle].altEnd = getValue(l_Alt)				
	rt_meas[cycle].speed = avg_speed / num_samples
	rt_meas[cycle].ldratio = (rt_meas[cycle].speed / 3.6 *delta_t) /(rt_meas[cycle].altStart - rt_meas[cycle].altEnd) -- speed(m/s)*duration / delta Altitude = waylength / delta alt
	rt_meas[cycle].vmin = Asp_min
	rt_meas[cycle].vmax = Asp_max
		if cycle <= max_cyc then 
			cycle = cycle +1 																						-- inc if meas. was succesfully finished																						-- tone "finished"
		end		
	reset_values()
	beep_finished()	
end




----------------------
--   run at start  --
----------------------

local function init()
--	arrayDef()
	for i=1, max_cyc do
		rt_meas[i] = {start =0, altStart=0, altEnd=0, speed=0, ldratio=0, vmin=0, vmax=0, dura=1}	
	end
end


--------------------
--   main app	  --
--------------------


local function run(event)

local speed = getValue(l_Airspeed)* metric

	l_time = getDateTime()							-- get time
	tmp_seconds = l_time.min*60 + l_time.sec		-- calc time in seconds
	arrayPrint()									-- show data
	
	if cycle >1 then								-- if minimum of one measurement is done: show average values
		disp_average(135,17)						-- at coord x,y
	end
	
	if getValue(sw_record) > 50 and record_actual == 0 and  cycle <= max_cyc then			-- check record start event (switch = "on" and no actual record situation)
		init_values(speed)																	-- init measurement
		beep_start()
	end
	

	if record_actual == 1 then 

		if  math.abs(speed - (avg_speed/num_samples)) > avg_speed*Vdelta/(100*num_samples) and num_samples > 1  then 		-- check if out of tolerance and abort	
			reset_values()
			beep_abort()			
		else																				-- else proceed measurement
			if (tmp_seconds - rt_meas[cycle].start) < delta_t then							-- check if record should proceed (meas.time; Vdelta; num_samples)
				num_samples = num_samples+1													-- one more sample
				avg_speed = avg_speed + speed
				if speed < Asp_min then Asp_min = speed end									-- store min/max values
				if speed > Asp_max then Asp_max = speed end
				
				if (tmp_seconds - rt_meas[cycle].start)%4 == 0 then 						-- cyclic signal "ongoing measurement"																	
					if pt_status == 0 then
						beep_ongoing()				
						pt_status = 1
					end
				else
					pt_status = 0
				end	
				
			else																			-- record successfully finished: store data
				meas_finished()			
			end
		end		
	end
	
end


return {init=init, run=run }